/**
 * 小程序
 * 组织生活
 */

import moment = require("moment");
import * as organizationalLifeData from "../../../data/organizationalLife";
import * as configEnum from "../../../config/enum";
import * as clientConfigEnum from "../../../config/clientEnum";
import { eccFormParam } from "../../../util/verificationParam";
import * as eccFormParamConfig from "../../../config/eccFormParamConfig";
import * as verificationEnumTools from "../../../util/verificationEnum";
import { checkChange, extractData } from "../../../util/piecemeal";
import * as splitResultConfig from "../../../config/splitResultConfig";
import * as partyMemberData from "../../../data/partyMember";
import { BizError } from "../../../util/bizError";
import { ERRORENUM } from "../../../config/errorEnum";

moment.suppressDeprecationWarnings = true;


/**
 * 组织生活列表
 * 小程序端 1.0
 * @param type 组织生活活动类型
 * @param partyActivityType 活动名称（党建活动类型）
 * @param branchName 支部名称
 * @returns data:[返回字段参考splitResultConfig文件中的moveAboutInfoConfig配置] 根据传入的活动状态返回不同的列表
 */
export async function getOrganizationalLifeList(type:number, partyActivityType:number, branchName) {
    /**将枚举值转为枚举文本 */
    let partyActivityTypeStr = verificationEnumTools.changeEnumValue(clientConfigEnum.TYPE, type);

    const NowMs = new Date().valueOf();
    /**拼接查询参数 */
    let selectParam = {};
    if (type != clientConfigEnum.TYPE.全部活动) {
        if (type == clientConfigEnum.TYPE.即将开展) selectParam = {"activityStartTime":{"$gt":NowMs }};
        else if (type == clientConfigEnum.TYPE.已经开展) selectParam = {"activityEndTime":{"$lt":NowMs }};
        else selectParam = {"$and":[ {"activityStartTime":{"$lt":NowMs }}, {"activityEndTime":{"$gt":NowMs}} ]}
    } else partyActivityTypeStr = "";
    if (partyActivityType) selectParam["partyActivityType"] = partyActivityType;
    if (branchName && branchName.length > 0) selectParam["branchName"] = {$in: branchName };

    let dbList = await organizationalLifeData.findOrganizationalLifeList(selectParam);

    let data = [];
    /**截取返回字段 */
    dbList.forEach(info => {
        let changeData:any = extractData(splitResultConfig.moveAboutInfoConfig, info, true);
        changeData.partyActivityType = verificationEnumTools.changeEnumValue(configEnum.ACTIVITYTYPE, info.partyActivityType);

        if (!partyActivityTypeStr) changeData.activityStatus = getActivityStatus(info.activityStartTime, info.activityEndTime);
        else changeData.activityStatus = partyActivityTypeStr;

        data.push(changeData);
    });
    
    return data;
}


/**
 * 创建组织生活活动
 * 小程序端 1.0
 * @param branchName 支部名称
 * @param param 创建组织生活活动表单
 * 表单参数说明查看config/eccFormParamConfig文件里的CreateActivityConfig配置
 * @returns {isSuccess: true} 是否创建成功
 */
export async function createOrganizationalLife(branchName, param) {
    /**校验表单参数 */
    eccFormParam("创建组织生活活动", eccFormParamConfig.CreateActivityConfig, param)
    if (branchName && branchName.length > 0) verificationEnumTools.eccEnumValue("创建组织生活活动", "支部名称", configEnum.BRANCHNAMETYPE, branchName);
    verificationEnumTools.eccEnumValue('创建组织生活活动的党建活动类型', 'partyActivityType', configEnum.ACTIVITYTYPE, param.partyActivityType);
    param.branchName = branchName;
    await organizationalLifeData.createActivity(param);

    return {isSuccess: true};
}


/**
 * 组织生活详细信息
 * 小程序端 1.0
 * @param oId 编号
 * @returns changeData:{返回字段参考splitResultConfig文件中的组织生活详细信息配置} 根据传入的组织生活oId返回详细信息
 */
export async function getOrganizationalLifeById(oId:number) {
    let activityInfo = await organizationalLifeData.findEventDetails(oId);

    /**截取返回字段 */
    let changeData:any = extractData(splitResultConfig.activityInfoConfig, activityInfo, true);
    changeData.title = verificationEnumTools.changeEnumValue(configEnum.ACTIVITYTYPE, activityInfo.partyActivityType);
    changeData.activityStatus = getActivityStatus(activityInfo.activityStartTime, activityInfo.activityEndTime);

    return changeData;
}


/**
 * 修改组织生活活动详情
 * 小程序端 1.0
 * @param oId 组织生活id 修改对应oId的组织生活信息
 * @param param 修改组织生活活动详情表单
 * 表单参数说明查看config/eccFormParamConfig文件里的UpdateActivityFormConfig配置
 * @returns {isSuccess: true}
 */
export async function updateOrganizationalLife(oId:number, param) {
    /**校验表单参数 */
    eccFormParam("组织生活活动表单修改", eccFormParamConfig.UpdateActivityFormConfig, param );
    verificationEnumTools.eccEnumValue('修改组织生活', 'partyActivityType', configEnum.ACTIVITYTYPE, param.partyActivityType);
    
    let activityInfo = await organizationalLifeData.findEventDetails(oId);
    if (!activityInfo) throw new BizError(ERRORENUM.未找到数据, `库中不存在id为${oId}的组织活动`);
    
    let changeList = checkChange(param, activityInfo);
    if ( !changeList.length ) throw new BizError(ERRORENUM.数据无更新, `${oId}数据无更新`);

    changeList.forEach(key => {
        activityInfo[key] = param[key];
    });

    await activityInfo.save();

    return {isSuccess:true};
}


/**
 * 创建/修改组织生活选择党员
 * 小程序端 1.0
 * 在使用枚举的情况下 参数中不能出现枚举以外的值
 * @param type 组织生活活动类型
 * @param name 党员姓名  用于输入党员姓名查询
 * @returns data:[返回字段参考splitResultConfig文件中的partyMemberConfig配置] 根据传入的type返回不同党员信息
 * 停用
 */
export async function getPartyMemberNameList(type:number, name:string) {
    let selectParam:any = {};
    let partyMemberList;

    if (type != clientConfigEnum.TYPE.全部活动) {
        selectParam.entryType = type;
        if(name.length) selectParam.name = name;
        partyMemberList = await partyMemberData.getPartyMemberListByEntryType(selectParam);
    } else {
        partyMemberList = await partyMemberData.findPartyMemberAll(selectParam);
    }

    let data = [];
    /**截取返回字段 */
    partyMemberList.forEach( info => {
        let changeData:any = extractData(splitResultConfig.partyMemberConfig, info, false);
        changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.PARTYPOSITIONS, info.partyPositions);
        data.push(changeData);
    })
    return data;
}


/**
 * 创建/修改组织生活选择党员
 * 小程序端 1.0
 * @returns data:[返回字段参考splitResultConfig文件中的organizationallifeConfig配置] 返回所有党员信息
 */
export async function getAllPartyMember() {
    let partyMemberList = await partyMemberData.findPartyMemberAll({});
    let data = [];
    /**截取返回字段 */
    partyMemberList.forEach( info => {
        let changeData:any = extractData(splitResultConfig.organizationallifeConfig, info, false);
        switch (changeData.entryType) {
            case 1:
                changeData.color = "#C62A23";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.PARTYPOSITIONSICON, info.partyPositions);
            break;
            case 2:
                changeData.color = "#3549B0";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
            case 3:
                changeData.color = "#E4B386";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
        }
        
        data.push(changeData);
    })

    return data;
}


/**
 * 删除组织生活活动
 * 小程序端 1.0
 * @param oId 活动id
 * @returns {isSuccess:true}
 */
export async function deleteOrganizationalLife(oId:number) {
    let activityInfo = await organizationalLifeData.findEventDetails(oId);
    if (!activityInfo) throw new BizError(ERRORENUM.未找到数据, `库中不存在id为${oId}的组织活动`);

    await organizationalLifeData.deleteActivity(oId);

    return {isSuccess:true};
}


/**
 * 组织生活活动状态
 * 小程序端 1.0
 * 活动状态 1：正在进行、2：即将开展、3：已经开展
 * @param activityStartTime 活动开始时间
 * @param activityEndTime 活动结束时间
 * @return status --活动状态
 */
function getActivityStatus(activityStartTime:number, activityEndTime:number) {
    let nowTime = new Date().valueOf();

    let startTime = moment(activityStartTime).format("YYYY.MM.DD HH:mm");
    let endTime = moment(activityEndTime).format("YYYY.MM.DD HH:mm");
    let underway = moment().isBetween(startTime, endTime); //true：正在进行

    let status;
    if (underway) {
        status = "正在进行";
    } else {
        if (nowTime < activityStartTime) {
            status = "即将开展";
        }
    
        if (nowTime > activityStartTime) {
            status = "已经开展";
        }
    }

    return status;
}