/**
 * 小程序
 * 党员基本信息
 */

import * as partyMemberData from "../../../data/partyMember";
import * as configEnum from "../../../config/enum";
import * as verificationEnumTools from "../../../util/verificationEnum";
import { eccFormParam } from "../../../util/verificationParam";
import * as eccFormParamConfig from "../../../config/eccFormParamConfig";
import { getPwdMd5, getToken } from "../../../tools/system";
import { extractData } from "../../../util/piecemeal";
import * as splitResultConfig from "../../../config/splitResultConfig";
import moment = require("moment");
import { BizError } from "../../../util/bizError";
import { ERRORENUM } from "../../../config/errorEnum";


/**
 * 获取党员名册
 * 小程序端 2.0
 * @param branchName 支部id
 * @returns 返回格式 {nameList:[返回字段参考splitResultConfig文件中的partyMemberConfig配置], partyMemberCount:{目前党员人数}}
 */
export async function getPartyMemberName(branchName) {
    let partyBranchName = verificationEnumTools.changeEnumValue(configEnum.BRANCHNAMETYPE, branchName);
    let partyMemberNum = await partyMemberData.groupFindPartyMemberCount({branchName:{$in: branchName }}); //党员总数

    let partyMemberCount = {
        key: partyBranchName + "目前党员人数",
        value: partyMemberNum
    };

    let partyMemberList = await partyMemberData.findPartyMemberAll({branchName:{$in: branchName }});
    let nameList = [];
    /**截取返回字段 */
    partyMemberList.forEach( info => {
        let changeData:any = extractData(splitResultConfig.partyMemberConfig, info, false);
        switch (changeData.entryType) {
            case 1:
                changeData.color = "#C62A23";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.PARTYPOSITIONSICON, info.partyPositions);
            break;
            case 2:
                changeData.color = "#3549B0";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
            case 3:
                changeData.color = "#E4B386";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
        }
        
        let time = moment(new Date()).format("YYYY");
        let partyJoiningTime = moment(info.partyJoiningTime).format("YYYY");
        changeData.partyStanding = parseInt(time) - parseInt(partyJoiningTime) + "年";
        changeData.department = verificationEnumTools.changeEnumValue(configEnum.DEDPARTMENT, changeData.department);
        nameList.push(changeData);
    })

    return {nameList, partyMemberCount};
}


/**
 * 创建党员基础信息
 * 小程序端 2.0
 * @param param 创建党员基础信息表单
 * 名册录入类型为正式党员 表单参数依据config/eccFormParamConfig文件里的CreateEntryTypeOneFormConfig配置
 * 名册录入类型为入党积极分子 表单参数依据config/eccFormParamConfig文件里的CreateEntryTypeTwoFormConfig配置
 * 名册录入类型为预备党员 表单参数依据config/eccFormParamConfig文件里的CreateEntryTypeThreeFormConfig配置
 * @param entryType 名册录入类型
 * @param branchName 支部名称
 * @return {isSuccess: true}
 */
 export async function createPartyMember(param, entryType, branchName) {
    /**校验表单参数 */
    if (entryType == configEnum.ENTRYTYPE.正式党员) {
        eccFormParam("创建正式党员录入", eccFormParamConfig.CreateEntryTypeOneFormConfig, param);
        if(!param.partyPositions) param.partyPositions.push(configEnum.PARTYPOSITIONSICON.党员);
        
        verificationEnumTools.eccEnumValue('正式党员录入党内职务', 'partyPositions', configEnum.PARTYPOSITIONSICON, param.partyPositions);
        param.entryType = configEnum.ENTRYTYPE.正式党员;
        param.partyState = configEnum.PARTYSTATE.在党;
        
    } else if (entryType == configEnum.ENTRYTYPE.入党积极分子) {
        eccFormParam("创建入党积极分子录入", eccFormParamConfig.CreateEntryTypeTwoFormConfig, param);
        if (!param.partyPositions || param.partyPositions.length < 1) param.partyPositions.push(configEnum.PARTYPOSITIONSICON.入党积极分子);
        // let partyPositions = [configEnum.PARTYPOSITIONSICON.入党积极分子];
        // if (param.partyPositions.toString() != partyPositions.toString()) throw new BizError(ERRORENUM.参数不符合规则, `${param.partyPositions} 创建入党积极分子的党内职务人员类型不符合规则`);
        
        verificationEnumTools.eccEnumValue('入党积极分子录入党内职务', 'partyPositions', configEnum.PARTYPOSITIONSICON, param.partyPositions);
        param.entryType = configEnum.ENTRYTYPE.入党积极分子;

    } else if (entryType == configEnum.ENTRYTYPE.预备党员) {
        eccFormParam("创建预备党员录入", eccFormParamConfig.CreateEntryTypeThreeFormConfig, param);
        if (!param.partyPositions || param.partyPositions.length < 1) param.partyPositions.push(configEnum.PARTYPOSITIONSICON.预备党员);
        // let partyPositions = [configEnum.PARTYPOSITIONSICON.预备党员];
        // if (param.partyPositions.toString()  != partyPositions.toString() ) throw new BizError(ERRORENUM.参数不符合规则, `${param.partyPositions} 创建预备党员的党内职务人员类型不符合规则`);
        
        verificationEnumTools.eccEnumValue('预备党员录入党内职务', 'partyPositions', configEnum.PARTYPOSITIONSICON, param.partyPositions);
        param.entryType = configEnum.ENTRYTYPE.预备党员;

    }

    verificationEnumTools.eccEnumValue('正式党员录入所属科室', 'department', configEnum.DEDPARTMENT, param.department);
    verificationEnumTools.eccEnumValue('正式党员录入行政职务', 'administrativePosition', configEnum.ADMINISTRATIVEPOSITION, param.administrativePosition);
    verificationEnumTools.eccEnumValue('正式党员录入文化程度', 'levelOfEducation', configEnum.DEGREE, param.levelOfEducation);
    verificationEnumTools.eccEnumValue('正式党员录入性别', 'partyActivityType', configEnum.SEX, param.sex);
    verificationEnumTools.eccEnumValue('正式党员名册录入类型', 'entryType', configEnum.ENTRYTYPE, entryType);

    if (branchName) param.branchName = branchName;

    let partyMemberInfo:any = await partyMemberData.createPartyMember(param);

    partyMemberInfo.loginId = param.phone;
    if(!param.pwd) partyMemberInfo.pwd = getPwdMd5(param.phone, param.phone.slice(5, 11));
    else partyMemberInfo.pwd = getPwdMd5(param.phone, param.pwd);
    partyMemberInfo.token = getToken(param.phone);
    partyMemberInfo.tokenMs = new Date().valueOf();

    await partyMemberInfo.save();

    return {isSuccess: true};
}


/**
 * 党员详情
 * 小程序端 2.0
 * @param id 党员id
 * @returns changeData:{返回字段参考splitResultConfig文件中的partyMemberDetailsConfig配置} 根据党员id返回党员详细信息
 */
export async function partyMemberDetails(id:number) {
    let partyMemberList = await partyMemberData.findPartyMemberById(id);
    let changeData:any = {};
    
    /**截取返回字段 */
    partyMemberList.forEach( info => {
        changeData = extractData(splitResultConfig.partyMemberDetailsConfig, info, false);
        switch (changeData.entryType) {
            case 1:
                changeData.color = "#C62A23";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.PARTYPOSITIONSICON, info.partyPositions);
            break;
            case 2:
                changeData.color = "#3549B0";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
            case 3:
                changeData.color = "#E4B386";
                changeData.partyPositions = verificationEnumTools.changeEnumValue(configEnum.ENTRYTYPE, info.entryType);
            break;
        }
        changeData.department = verificationEnumTools.changeEnumValue(configEnum.DEDPARTMENT, info.department);
        changeData.sex = verificationEnumTools.changeEnumValue(configEnum.SEX, info.sex);
        changeData.administrativePosition = verificationEnumTools.changeEnumValue(configEnum.ADMINISTRATIVEPOSITION, info.administrativePosition);
        changeData.partyState = verificationEnumTools.changeEnumValue(configEnum.PARTYSTATE, info.partyState);
        changeData.levelOfEducation = verificationEnumTools.changeEnumValue(configEnum.DEGREE, info.levelOfEducation);
        
        changeData.birthDate = moment(info.birthDate).format("YYYY年MM月DD日");
        changeData.partyJoiningTime = moment(info.partyJoiningTime).format("YYYY年MM月DD日");
        //changeData.study 学习相关数据，暂时写死假数据
        changeData.study = [ 
            {key:"学习排名", value:getValueByLength(1)},
            {key:"学习课时", value:getValueByLength(2)},
            {key:"学习进度", value:getValueByLength(2) + "%"}
        ];

        let time = moment(new Date()).format("YYYY");
        let partyJoiningTime = moment(info.partyJoiningTime).format("YYYY");
        changeData.partyStanding = parseInt(time) - parseInt(partyJoiningTime) + "年";
        
    })

    return changeData;
}


/**
 * 删除党员
 * 小程序端 2.0
 * @param id 党员id
 * @returns {isSuccess:true}
 */
 export async function deletePartyMember(id:number) {
    let activityInfo = await partyMemberData.findPartyMemberById(id);
    if (!activityInfo) throw new BizError(ERRORENUM.未找到数据, `库中不存在id为${id}党员信息`);

    await partyMemberData.deletePartyMember(id);

    return {isSuccess:true};
}




/**
 * 随机数生成假数据
 * @param length 假数据长度
 * @returns 
 */
function getValueByLength(length) {
    let i = 1;
    getPlaByLength(length, () => {
        i= i * 10;
    });
    return Math.floor(Math.random() * i);
}

function getPlaByLength(num, callback) {
    for (let i =0; i < num; i++) {
        callback();
    }
}
